#include "SatScan.hh"
#include "FilterDB.hh"
#include "ParseLine.hh"
#include "Dacc.hh"
#include <vector>

using namespace std;

EXECDAT(SatScan)

//======================================  Constructor
SatScan::SatScan(int argc, const char* argv[])
  : DatEnv(argc, argv), MonServer("SatScan"), mConfig(mIn), mLast(0), 
    mCount(10), mOsc(mIn), mTrend("SatScan", Trend::kMinute), 
    mAlarm("SatScan")
{
    bool syntax(false);
    for (int i=1 ; i<argc ; i++) {
        string argi = argv[i];
	if (argi == "-conf") {
	    mConfFile = argv[++i];
	} else if (argi == "-osc") {
	    mOscFile = argv[++i];
	} else if (isDatEnvArg(argv[i])) {
	    i++;
	} else {
	    cerr << "Unknown argument: " << argi << endl;
	    syntax = true;
	    break;
	}
    }
    if (mConfFile.empty()) syntax = true;
    if (syntax) {
        finish();
	cerr << "Command syntax: " << endl;
	cerr << "SatScan -conf <file> <DatEnv-arguments>" << endl;
	return;
    }
    ReadConfig();
    getDacc().setIgnoreMissingChannel(true);
    getDacc().setStride(6.0);
    mConfig.setDebug(Debug());
    if (!mOscFile.empty()) mOsc.readConfig(mOscFile.c_str());
    if (Debug() > 1) mIn.list(cout);

    //----------------------------------  Set up the alarm descriptor
    mAlDat = AlarmData("SatScan", "Saturation", Interval(60.0), 4, 
		       "Saturation in $1 ");
    mAlDat.jamFlags(AlarmData::kReTrigger);
    mAlarm.defineAlarm(mAlDat);
}

//======================================  Constructor
void 
SatScan::ReadConfig(void) {

    //----------------------------------  Loop over configuration lines
    ParseLine pl(mConfFile.c_str());
    while (pl.getLine() >= 0) {
        int nArg = pl.getCount();
	if (!nArg) continue;

	//-----------------------------
	string pdName = pl[0];
	if (pdName.substr(2, 5) != ":LSC-") {
	    cerr << "Bad PD name: " << pdName << endl;
	    continue;
	}
	string ifo(pdName.substr(0, 2));
	string port(pdName.substr(7));
	string::size_type inx = port.size()-1;
	int pd = -1;
	if (port[inx] >= '0' && port[inx] <= '9') {
	    pd = port[inx] - '0';
	    port.erase(inx);
	}
	mList.push_back(SatChan(ifo.c_str(), port.c_str(), pd));

	for (int i=1 ; i<nArg ; i++) {
	    string argi = pl[i];
	    if (argi == "-repeat") {
	        long rpt = strtol(pl[++i], 0, 0);
		mList.back().setRepCount(rpt);
	    } else if (argi == "-when") {
		mList.back().setOscCond(pl[++i]);
	    } else {
	        cerr << "Unidentified parameter: " << argi << endl;
	    }
	}

	string chan = mList.back().getChannelI();
	getDacc().addChannel(chan.c_str());
	chan = mList.back().getChannelQ();
	getDacc().addChannel(chan.c_str());
    }

    for (pd_iter i=mList.begin() ; i != mList.end() ; i++) {
        i->setCalibrate(mConfig);
	i->refHistoryI().setMaxLen(43200.0);
	string chan = i->getChannelI() + "_OverFlow";
	serveData(chan.c_str(), &(i->refHistoryI()));
	mTrend.addChannel(chan.c_str());

	i->refHistoryQ().setMaxLen(43200.0);
	chan = i->getChannelQ() + "_OverFlow";
	serveData(chan.c_str(), &(i->refHistoryQ()));
	mTrend.addChannel(chan.c_str());
    }
}

//======================================  Destructor
SatScan::~SatScan(void) {
}

//======================================  Destructor
void
SatScan::Attention(void) {
    if (Debug()) cout << "PSLmon: Attention Interrupt" << endl;
    MonServer::Attention();
}

//======================================  Process data
void 
SatScan::ProcessData(void) {
    const char* phazNam[2] = {"_Q", "_I"};

    //----------------------------------  Check the configuration
    Time tStart = getDacc().getFillTime();

    mConfig.getConfig(tStart);
 
    //----------------------------------  Loop over the channels
    for (pd_iter i=mList.begin() ; i != mList.end() ; i++) {

        //------------------------------  Get IFO name
	string Ifo(i->getIfo());

	//------------------------------  Test the condition
	bool inseg(true);
	if (!mOscFile.empty()) {
	    const char* cond = i->getOscCond();
	    if (cond && *cond && !mOsc.satisfied(cond)) inseg = false;
	}

	//------------------------------  Get data
	int nRpt = i->getRepCount();
	string chNameI(i->getChannelI());
        TSeries* chanI = getDacc().refData(chNameI.c_str());
	if (!chanI) {
	    cout << "Unable to get channel " << chNameI << endl;
	    continue;
	} else if (chanI->isEmpty()) {
	    cout << "Channel " << chNameI << " is empty." << endl;
	    continue;
	}

	string chNameQ(i->getChannelQ());
        TSeries* chanQ = getDacc().refData(chNameQ.c_str());
	if (!chanQ) {
	    cout << "Unable to get channel " << chNameQ << endl;
	    continue;
	} else if (chanQ->isEmpty()) {
	    cout << "Channel " << chNameQ << " is empty." << endl;
	    continue;
	}
	Interval dt = chanI->getInterval();

        //------------------------------  Loop over phases
	for (int phaz=0 ; phaz < 2 ; phaz++) {
	    TSeries  PdX;
	    if (!phaz) PdX = i->getPdQ(*chanQ, *chanI);
	    else       PdX = i->getPdI(*chanQ, *chanI);
	    long N = PdX.getNSample();
	    if (!N) continue;

	    if (mCount && Debug()) {
	        const int nSnip(32);
	        cout << endl;
		cout << "Dump of first " << nSnip << " samples of " 
		     << i->getBaseName() << phazNam[phaz] << endl;
		PdX.extract(tStart, double(nSnip)*PdX.getTStep()).Dump(cout);
	    }

	    //-------------------------------  scan for repetitions
	    int   xCount(0);
	    float totErr(0.0);
	    Time  Start(0);
	    if (inseg) {
	        string chName;
		if (!phaz) chName = chNameQ;
		else       chName = chNameI;
		
	        const float* pF=reinterpret_cast<const float*>(PdX.refData());
		int xLast = int(floor(*pF++ + 0.5));
		for (long j=1 ; j<N ; j++) {
		    int  x = int(floor(*pF++ + 0.5));
		    if (x == xLast) {
		        if (!xCount++) Start=PdX.getBinT(j-1);
		    } else if ((nRpt && xCount >=nRpt) || xLast == 32767 
			                               || xLast == -32768) {
		        xCount++;
		        if (Debug() > 1) {
			    cout << chName << " " << Start << " " << xCount 
				 << " " << xLast << endl;
			}
			totErr += xCount;
			xCount = 0;
			xLast  = x;
		    } else {
			xCount = 0;
			xLast  = x;
		    }
		}
		if ((nRpt && xCount >= nRpt) || xLast == 32767 
		                             || xLast == -32768) {
		    xCount++;
		    if (Debug() > 1) {
			cout << chName << " " << Start << " " << xCount 
			     << " " << xLast << endl;
		    }
		    totErr += xCount;
		}

		string trendName(chName + "_OverFlow");
		mTrend.trendData(trendName.c_str(), tStart, double(totErr));
		if (totErr) {
		    AlarmHandle han;
		    AlarmData al("SatScan", "Saturation", 60.0, 4, "", chName);
		    mAlarm.setAlarm(al, han);
		}
	    }

	    //---------------------------------  Update the history
	    if (!phaz) {
	        FixedLenTS& iPhazTS(i->refHistoryI());
	        if (iPhazTS.isEmpty()) iPhazTS.padUntil(tStart, dt);
	        iPhazTS.fixedAppend(tStart, dt, &totErr);
	    } else {
	        FixedLenTS& qPhazTS(i->refHistoryQ());
	        if (qPhazTS.isEmpty()) qPhazTS.padUntil(tStart, dt);
	        qPhazTS.fixedAppend(tStart, dt, &totErr);
	    }
	}
    }
    
    if (mCount) mCount--;
    mLast = getDacc().getCurrentTime();
}
